// =======================================================================
// PRE-FLIGHT CHECK VARIABLES
// =======================================================================

var _checkDeviceStatus = null;
var _isSatSupported = null;
var _isPreviousJob = null;
var _checkVersion = null;

var _commError = false;

// =======================================================================
// GLOBAL VARIABLES
// =======================================================================

var Version = 2;

var API = "http://api.submarine.gg/TaxiRank/V1.svc/";
//var API = "http://localhost:9275/V1.svc/";

var UUID;
var ClientID;

var availableDrivers = null;

var curStatus = 0;

var Lat = null;
var Lon = null;

var DateTime;
var destLat = null;
var destLon = null;

var Map = null;

var addUserMarkerTicker;

var checkDate = null;

var destinationMarker = null;
var destinationMarkerIcon;

var driverMarker = null;
var driverMarkerIcon;
var driverTicker;

var checkJourneyStatusTicker;

var JourneyID;

var roadName;
var parishName;
var _isRoadReady = false;

var FromLn1;
var FromLn2;
var FromLn3;

var ToLn1;
var ToLn2;
var ToLn3;

var NumPeople;
var DisabilityAccess;

var countAvailableDriversTicker;

var Rec;
var Notes;

var telephoneNumber = false;
var homeAddressLn1 = false;
var homeAddressLn2 = false;
var homeAddressLn3 = false;

// =======================================================================

// =======================================================================
// PRE-FLIGHT CHECK FUNCTIONS
// =======================================================================

function checkDeviceStatus() {

    $.ajax({
        url: API + "getDeviceStatus/" + UUID,
        success: function (data) {
            _checkDeviceStatus = data.getDeviceStatusResult;
        },
        error: function () {
            _commError = true;
        }
    });
}

function checkVersion() {

    $.ajax({
        url: API + "getVersion/",
        success: function (data) {
            _checkVersion = data.getVersionResult;
        },
        error: function () {
            _commError = true;
        }
    });
}

function isSatSupported() {

    _isSatSupported = (navigator.geolocation) ? true : false;
}

function isPreviousJob() {

    $.ajax({
        url: API + "isPreviousClientJob/" + UUID,
        success: function (data) {

            $('#Loading').hide();
            $('#Home').show();

            if (data.isPreviousClientJobResult != -1) {
                _isPreviousJob = true;

                $('#Home').hide();

                $('#MadeConnection').show();
                $('#cancelJob').show();

                if (Map == null) {

                    Map = L.map("MadeConnection", { zoomControl: false });

                    L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
                        attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> Contributors'
                    }).addTo(Map);
                }

                JourneyID = data.isPreviousClientJobResult;

                checkJourneyStatusTicker = setInterval(checkJourneyStatus, 1000);
            }
            else {
                _isPreviousJob = false;
            }
        },
        error: function () {
            _commError = true;
        }
    });
}

// =======================================================================
// STARTUP OPERATIONS
// =======================================================================

document.addEventListener("deviceready", init, false);

function init() {

    checkVersion();

    var versionTicker = setInterval(versionReady, 1000);

    function versionReady() {

        if (_checkVersion != null) {

            clearInterval(versionTicker);

            if (_checkVersion > Version) {

                hardError("Version Mismatch", "Unfortunately you're running an out of date version of the app.<br><br>Please download a new version from your device's app store.");
            }
            else {

                document.getElementById('btnRegisterDevice').addEventListener('click', RegisterDevice, false);
                document.getElementById('btnActivateDevice').addEventListener('click', ActivateDevice, false);

                UUID = (device.platform == "Win32NT") ? intel.xdk.device.uuid : device.uuid;

                checkDeviceStatus();
                isSatSupported();

                var preFlightTicker = setInterval(isReady, 1000);

                function isReady() {

                    if (_commError) {

                        clearInterval(preFlightTicker);
                        showCommError();
                    }
                    else {
                        if (_checkDeviceStatus != null && _isSatSupported != null) {

                            $('#LoadingText').html("Finalising Checks");

                            clearInterval(preFlightTicker);

                            if (_checkDeviceStatus == 1) {
                                $('#RegisterDevice').show();
                                $('#Loading').hide();
                            }
                            else if (_checkDeviceStatus == 2) {
                                hardError("Device Banned", "Unfortunately your device has been banned, this could be due to a number of reasons. <br><br>If you believe this to be a mistake, please contact Submarine.");
                            }
                            else if (_checkDeviceStatus == 3) {
                                $('#ActivateDevice').show();
                                $('#Loading').hide();
                            }
                            else {
                                $('#ActivateDevice').hide();

                                userMarkerIcon = L.icon({

                                    iconUrl: 'assets/Start.png',

                                    iconSize: [75, 75],
                                    iconAnchor: [37.5, 75]
                                });

                                destinationMarkerIcon = L.icon({

                                    iconUrl: 'assets/End.png',

                                    iconSize: [75, 75],
                                    iconAnchor: [37.5, 75]
                                });

                                driverMarkerIcon = L.icon({

                                    iconUrl: 'assets/Taxi.png',

                                    iconSize: [75, 75],
                                    iconAnchor: [37.5, 75]
                                });

                                $('#LoadingText').html("Checking for Previous Jobs");

                                getTelephone();
                                getAddress();
                                isPreviousJob();

                                uidToID();
                            }
                        }
                    }
                }
            }
        }
    }
}

// =======================================================================
// EVERYTHING ELSE
// =======================================================================

function uidToID() {

    $.ajax({
        url: API + "uidToID/" + UUID,
        success: function (data) {
            ClientID = data.uidToIDResult;
        },
        error: function () {
            showCommError();
        }
    });
}

function countAvailableDrivers() {

    $.ajax({
        url: API + "countAvailableDrivers/" + Date.now(),
        success: function (data) {
            availableDrivers = data.countAvailableDriversResult;
        },
        error: function () {
            availableDrivers = 0;
        }
    });
}

function hardError(Title, Error) {

    $('.Screen').hide();
    $('#Loading').hide();

    if ($('#hardError').length) { $('#hardError').html("<p>" + Error + "</p>"); }
    else {

        $('body').append('<div id="hardError" class="Screen" ></div>');
        $('#hardError').html("<h1>" + Title + "</h1><br><p>" + Error + "</p>");
    }
}

function getRoadFromGPS(_Lat, _Lon) {

    $('#LoadingText').html("Reverse Geocoding");

    $.ajax({
        url: API + "getRoadFromGPS/" + _Lat + "/" + _Lon,
        success: function (data) {
            roadName = data.getRoadFromGPSResult.m_Item1;
            parishName = data.getRoadFromGPSResult.m_Item2;
            _isRoadReady = true;
        },
        error: function (data) {
            _isRoadReady = true;
        }
    });
}

function goToPage(From, To) {

    $('#' + From).hide();
    $('#softError').hide();
    $('.Screen').hide();

    $('#' + To).show();

    if (To == "FindATaxi"  || To == "MakeBooking") {

        countAvailableDrivers();

        if (To == "MakeBooking") {
            availableDrivers = 1;
            $('#LoadingText').html("Loading");
        }
        else {
            $('#LoadingText').html("Getting Drivers");
        }

        countAvailableDriversTicker = setInterval(checkDrivers, 1000);

        $('#Loading').show();

        function checkDrivers() {

            if (availableDrivers != null) {

                clearInterval(countAvailableDriversTicker);

                $('#Loading').hide();

                if (availableDrivers == 0) {

                    availableDrivers = null;

                    displayMessage("Unfortunately no drivers are available at this time.", "Uh Oh!");
                    $('.Screen').hide();
                    $('#FindATaxi').hide();
                    $('#Home').show();
                }
                else {
                    availableDrivers = null;

                    if (telephoneNumber) {

                        if (To != "MakeBooking") {

                            $('#Loading').show();
                            $('#LoadingText').html("Locating your Position");

                            var X = $('#tbAddrFromLn2').val();

                            if (_isSatSupported && X.length == 0) {

                                getGPS();

                                var attemptCounter = 0;

                                function hasGPS() {

                                    if (attemptCounter >= 10) {

                                        clearInterval(addUserMarkerTicker);

                                        $('#addDetailsMessage').html("We unfortunately couldn't obtain your location via GPS, please enter it manually.");
                                        goToPage("FindATaxi", "AdditionalDetails");

                                        Lat = Lon = 0;

                                        $('#Loading').hide();
                                    }

                                    attemptCounter++;

                                    if (Lat != null && Lon != null && _isRoadReady) {

                                        $('#addDetailsMessage').html("");

                                        clearInterval(addUserMarkerTicker);

                                        $('#tbAddrFromLn2').val(roadName);
                                        $('#tbAddrFromLn3').val(parishName);
                                        //$('#tbAddrFromLn3').val(null);

                                        goToPage("FindATaxi", "AdditionalDetails");

                                        $('#Loading').hide();

                                        _isRoadReady = false;
                                    }
                                }

                                addUserMarkerTicker = setInterval(hasGPS, 500);
                            }
                            else {

                                $('#addDetailsMessage').html("We unfortunately couldn't obtain your location via GPS, please enter it manually.");
                                goToPage("FindATaxi", "AdditionalDetails");

                                $('#Loading').hide();
                            }

                            $('#ReqTaxi').show();
                            $('#AdvBooking').hide();
                        }
                        else {

                            $('#addDetailsMessage').html("Please enter the pickup location.");
                            goToPage("FindATaxi", "AdditionalDetails");

                            $('#ReqTaxi').hide();
                            $('#AdvBooking').show();

                            $('#Loading').hide();
                        }
                    }
                    else {
                        displayMessage("You appear to be missing a telephone number, please review this option under 'App Settings'", "Uh Oh!");
                        $('#Home').show();
                    }
                }
            }
        }
    }
    else if (To == "Bookings") {

        $('#BackBar').attr('onclick', 'goToPage("Bookings", "Home");');
        $('#BackBar').show();

        $('#LoadingText').html("Getting Bookings");
        $('#Loading').show();

        $.ajax({
            url: API + "getOpenBookings/" + ClientID + "/",
            success: function (data) {

                if (data.getOpenBookingsResult.length > 0) {

                    $('#Bookings').html("<p>(Note: Bookings in green have been accepted)</p><br><ul id='bookingList' style='background: rgba(0, 0, 0, 0.8);padding: 5px;'></ul>");

                    for (var i = 0; i < data.getOpenBookingsResult.length; i++) {

                        if (data.getOpenBookingsResult[i].m_Item4 == 11) {
                            $('#bookingList').append("<li style='list-style: none; margin-bottom: 25px; margin-bottom: 20px; padding: 5px; text-align: center; border: 1px solid #FFFFFF; border-radius: 5px;' onclick=\"selectBooking('" + data.getOpenBookingsResult[i].m_Item1 + "');\" >" + data.getOpenBookingsResult[i].m_Item2 + " at " + data.getOpenBookingsResult[i].m_Item3 + "</li>");
                        }
                        else if (data.getOpenBookingsResult[i].m_Item4 == 12) {
                            $('#bookingList').append("<li style='list-style: none; margin-bottom: 25px; margin-bottom: 20px; padding: 5px; text-align: center; border: 1px solid #FFFFFF; border-radius: 5px;' onclick=\"selectBooking('" + data.getOpenBookingsResult[i].m_Item1 + "');\" class='Green' >" + data.getOpenBookingsResult[i].m_Item2 + " at " + data.getOpenBookingsResult[i].m_Item3 + "</li>");
                        }
                    }

                    $('#Loading').hide();
                }
                else {

                    $('#Loading').hide();
                    goToPage('Bookings', 'Home');

                    displayMessage("You don't appear to have any open bookings, you can create one under the 'Find A Taxi' option.", "No Bookings");
                }

                
            },

            error: function () {

                $('#Loading').hide();
                goToPage('Bookings', 'Home');

                displayMessage("A problem occurred whilst attempting to retrieve your bookings, please try again later.", "Uh Oh!");
            }
        });
    }
    else if (To == "AdvancedBooking") {

        FromLn1 = $('#tbAddrFromLn1').val();
        FromLn2 = $('#tbAddrFromLn2').val();
        FromLn3 = $('#tbAddrFromLn3').val();

        ToLn1 = $('#tbAddrToLn1').val();
        ToLn2 = $('#tbAddrToLn2').val();
        ToLn3 = $('#tbAddrToLn3').val();

        NumPeople = $('#ddlPeople').val();
        DisabilityAccess = ($('#ddlDisAcc').val() == "Yes") ? true : false;

        Rec = $('#tbRec').val();
        Notes = $('#tbNotes').val();

        if (FromLn1.length > 0 && FromLn2.length > 0 && FromLn3.length > 0 && ToLn1.length > 0 && ToLn2.length > 0 && ToLn3.length > 0 && Rec.length > 0) {

            $('#BackBar').attr('onclick', 'goToPage("AdvancedBooking", "AdditionalDetails");');

        }
        else {

            $('#' + From).show();
            $('#AdvancedBooking').hide();

            displayMessage("You appear to be missing some required fields.", "Uh Oh!");

            if ($('#tbAddrFromLn1').val().length == 0) {
                $('#tbAddrFromLn1').css('border', '2px solid #FF0000');
            }
            if ($('#tbAddrFromLn2').val().length == 0) {
                $('#tbAddrFromLn2').css('border', '2px solid #FF0000');
            }
            if ($('#tbAddrFromLn3').val() == null) {
                $('#tbAddrFromLn3').css('border', '2px solid #FF0000');
            }

            if ($('#tbAddrToLn1').val().length == 0) {
                $('#tbAddrToLn1').css('border', '2px solid #FF0000');
            }
            if ($('#tbAddrToLn2').val().length == 0) {
                $('#tbAddrToLn2').css('border', '2px solid #FF0000');
            }
            if ($('#tbAddrToLn3').val() == null) {
                $('#tbAddrToLn3').css('border', '2px solid #FF0000');
            }

            if ($('#tbRec').val().length == 0) {
                $('#tbRec').css('border', '2px solid #FF0000');
            }
        }
    }
    else if (To == "POI") {

        $('#Loading').show();
        $('#LoadingText').html("Getting POIs");

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("POI", "Home");');

        $.ajax({
            url: API + "getPOI/",
            success: function (data) {

                $('#POI').html("<h1 style='color: #F3C400; font-size: 1.2em; text-align: center; padding: 10px;'>Select a POI</h1><br><ul id='poiList' style='background: rgba(0, 0, 0, 0.8);padding: 5px;'></ul>");

                for (var i = 0; i < data.getPOIResult.length; i++) {

                    $('#poiList').append("<li style='list-style: none; margin-bottom: 25px; margin-bottom: 20px; padding: 5px; text-align: center; border: 1px solid #FFFFFF; border-radius: 5px;' onclick=\"selectPOI('" + data.getPOIResult[i].m_Item1 + "');\" >" + data.getPOIResult[i].m_Item2 + "</li>");
                }

                $('#Loading').hide();
            },
            error: function () {

                $('#Loading').hide();

                goToPage("POI", "Home");
                displayMessage("Unfortunately we could not retrieve a list of POIs due to a communication problem. Please try again later.", "Uh Oh!");
            }
        });
    }
    else if (To == "AddressBook") {

        $('#Loading').show();
        $('#LoadingText').html("Getting Addresses");

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("AddressBook", "Home");');

        $.ajax({
            url: API + "getAddressBook/" + ClientID + "/",
            success: function (data) {

                $('#AddressBook').html("<h1 style='color: #F3C400; font-size: 1.2em; text-align: center; padding: 10px;'>Address Book</h1><br><ul id='addressList' style='background: rgba(0, 0, 0, 0.8);padding: 5px;'></ul>");

                $('#addressList').append("<li style='list-style: none; margin-bottom: 25px; margin-bottom: 20px; padding: 5px; text-align: center; border-radius: 5px;' class='Green' onclick=\"\goToPage('AddressBook', 'AddAddress');\" >Add a new Address</li>");

                for (var i = 0; i < data.getAddressBookResult.length; i++) {

                    $('#addressList').append("<li style='list-style: none; margin-bottom: 25px; margin-bottom: 20px; padding: 5px; text-align: center; border: 1px solid #FFFFFF; border-radius: 5px;' onclick=\"selectAddress('" + data.getAddressBookResult[i].m_Item1 + "');\" >" + data.getAddressBookResult[i].m_Item2 + "</li>");
                }

                $('#Loading').hide();
            },
            error: function () {

                $('#Loading').hide();

                goToPage("AddressBook", "Home");
                displayMessage("Unfortunately we could not retrieve your list of addresses due to a communication problem. Please try again later.", "Uh Oh!");
            }
        });
    }
    else if (To == "AddAddress") {

        $('#tbAddAddrName').val("");
        $('#tbAddAddrLn1').val("");
        $('#tbAddAddrLn2').val("");
        $('#tbAddAddrLn3').val("");

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("AddAddress", "AddressBook");');
    }
    else if (To == "TakeMeHome") {

        if (homeAddressLn1 == null || homeAddressLn2 == null || homeAddressLn3 == null) {

            displayMessage("You appear to be missing a home address, please review this option under 'App Settings'", "Uh Oh!");

            goToPage('Home', 'Home');
        }
        else {
            $('#tbAddrToLn1').val(homeAddressLn1);
            $('#tbAddrToLn2').val(homeAddressLn2);
            $('#tbAddrToLn3').val(homeAddressLn3);

            goToPage('Home', 'FindATaxi');
        }
    }
    else if (To == "AdditionalDetails") {

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("AdditionalDetails", "Home");');
    }
    else if (To == "About") {

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("About", "Home");');
    }
    else if (To == "HowItWorks") {

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("HowItWorks", "Home");');
    }
    else if (To == "SettingsScreen") {

        $('#tbTelephone').val(telephoneNumber);

        $('#tbTMHAddrLn1').val(homeAddressLn1);
        $('#tbTMHAddrLn2').val(homeAddressLn2);
        $('#tbTMHAddrLn3').val(homeAddressLn3);

        $('#BackBar').show();
        $('#BackBar').attr('onclick', 'goToPage("SettingsScreen", "Home");');
    }

    if (From == "About") {
        $('#BackBar').hide();
    }
    else if (From == "Bookings") {
        $('#BackBar').hide();
    }
    else if (From == "HowItWorks") {
        $('#BackBar').hide();
    }
    else if (From == "AdditionalDetails" && To != "AdvancedBooking") {
        $('#BackBar').hide();
    }
    else if (From == "POI") {
        $('#BackBar').hide();
    }
    else if (From == "AddressBook" && To != "AddAddress") {
        $('#BackBar').hide();
    }
    else if (From == "AddAddress" && To != "AddressBook") {
        $('#BackBar').hide();
    }
    else if (From == "SettingsScreen") {
        $('#BackBar').hide();
    }
    else if (From == "DriverList") {
        $('#BackBar').hide();
    }
}

function displayMessage(Message, Title) {

    if (device.platform == "Win32NT") {
        intel.xdk.notification.alert(Message, Title, "OK");
    }
    else {
        navigator.notification.alert(Message, null, Title, "OK");
    }
}

function bookTaxi() {

    FromLn1 = $('#tbAddrFromLn1').val();
    FromLn2 = $('#tbAddrFromLn2').val();
    FromLn3 = $('#tbAddrFromLn3').val();

    ToLn1 = $('#tbAddrToLn1').val();
    ToLn2 = $('#tbAddrToLn2').val();
    ToLn3 = $('#tbAddrToLn3').val();

    NumPeople = $('#ddlPeople').val();
    DisabilityAccess = ($('#ddlDisAcc').val() == "Yes") ? true : false;

    Rec = $('#tbRec').val();
    Notes = $('#tbNotes').val();

    if (FromLn1.length > 0 && FromLn2.length > 0 && FromLn3.length > 0 && ToLn1.length > 0 && ToLn2.length > 0 && ToLn3.length > 0 && Rec.length > 0) {

        $('#AdditionalDetails').hide();
        $('#BackBar').hide();
        $('#Loading').show();
        $('#LoadingText').html("Pairing you with a Driver");

        $.ajax({
            url: API + "getClosestDriverList/" + Lat + "/" + Lon + "/" + NumPeople + "/" + DisabilityAccess,
            success: function (data) {

                if (data.getClosestDriverListResult.length == 0) {
                    displayMessage("Unfortunately no drivers are available at this time.", "Uh Oh!");
                    $('.Screen').hide();
                    $('#Loading').hide();
                    $('#Home').show();
                }
                else {

                    $('#DriverList').show();

                    $('#BackBar').show();
                    $('#BackBar').attr('onclick', 'goToPage("DriverList", "Home");');

                    $('#DriverList').html("<h1 style='color: #F3C400; font-size: 1.2em; text-align: center; padding: 10px;'>Select Your Driver</h1><br><p style='font-size: 1em; text-align: center;' >(Ordered Closest to Farthest Away)</p><br><ul id='drvList' style='background: rgba(0, 0, 0, 0.8);padding: 5px;'></ul>");

                    for (var i = 0; i < data.getClosestDriverListResult.length; i++) {

                        $('#drvList').append("<li style='list-style: none; margin-bottom: 25px; margin-bottom: 20px; padding: 5px; text-align: center; border: 1px solid #FFFFFF; border-radius: 5px;' onclick=\"selectDriver('" + data.getClosestDriverListResult[i].m_Item1 + "');\" >" + data.getClosestDriverListResult[i].m_Item2 + "</li>");
                    }

                    $('#Loading').hide();
                }
            },
            error: function () {
                showCommError();
            }
        });
    }
    else {
        displayMessage("You appear to be missing some required fields.", "Uh Oh!");

        if ($('#tbAddrFromLn1').val().length == 0) {
            $('#tbAddrFromLn1').css('border', '2px solid #FF0000');
        }
        if ($('#tbAddrFromLn2').val().length == 0) {
            $('#tbAddrFromLn2').css('border', '2px solid #FF0000');
        }
        if ($('#tbAddrFromLn3').val() == null) {
            $('#tbAddrFromLn3').css('border', '2px solid #FF0000');
        }

        if ($('#tbAddrToLn1').val().length == 0) {
            $('#tbAddrToLn1').css('border', '2px solid #FF0000');
        }
        if ($('#tbAddrToLn2').val().length == 0) {
            $('#tbAddrToLn2').css('border', '2px solid #FF0000');
        }
        if ($('#tbAddrToLn3').val() == null) {
            $('#tbAddrToLn3').css('border', '2px solid #FF0000');
        }

        if ($('#tbRec').val().length == 0) {
            $('#tbRec').css('border', '2px solid #FF0000');
        }
    }
}

function selectDriver(driverID) {

    checkDate = null;

    $('#Loading').show();
    $('#LoadingText').html("Pairing you with a Driver");
    $('#DriverList').hide();

    $.ajax({

        type: "POST",
        url: API + "createJourney/",
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        data: JSON.stringify({ "ClientUID": UUID, "FrLn1": FromLn1, "FrLn2": FromLn2, "FrLn3": FromLn3, "ToLn1": ToLn1, "ToLn2": ToLn2, "ToLn3": ToLn3, "Rec": Rec, "Notes": Notes, "DriverID": driverID, "People": NumPeople, "DisAcc": DisabilityAccess, "Date": null, "Time": null }),
        success: function (data, status, jqXHR) {

            if (data.createJourneyResult < 1) {

                if (data.createJourneyResult == 0.1) {

                    $('#Home').show();
                    $('#Loading').hide();
                    displayMessage("Unfortunately we could not create your job due to a permission or status conflict.", "Hmm...");
                }
                else if (data.createJourneyResult == 0.2) {

                    $('#DriverList').show();
                    bookTaxi();
                    $('#Loading').hide();
                    displayMessage("Unfortunately this driver is no longer available, please select another driver from the list.", "Uh Oh!");
                }
                else {

                    $('#Home').show();
                    $('#Loading').hide();
                    displayMessage("Something went wrong but we don't know why, please try again shortly.", "Uh Oh!");
                }
            }
            else {

                $('#BackBar').hide();

                JourneyID = data.createJourneyResult;

                $('#LoadingText').html("Looking for the Driver");

                checkJourneyStatusTicker = setInterval(checkJourneyStatus, 1000);
            }
        },
        error: function () {
            showCommError();
        }
    });
}

function checkJourneyStatus() {

    DateTime = new Date();
    DateTime.setMinutes(DateTime.getMinutes() - 15);

    $.ajax({

        url: API + "checkJourneyStatus/" + JourneyID + "/" + Date.now(),
        success: function (data) {

            LastUpdate = new Date(Date.parse(data.checkJourneyStatusResult.m_Item2));

            /*if (LastUpdate < DateTime.getTime() && !$('#cancelJob').is(':visible')) {
                $('#cancelJob').show();
                displayMessage("The driver is taking longer than expected, if you'd like to cancel this and request a new driver then please use the button provided on the map screen.", "Hmm....");
            }
            else {*/

            if (data.checkJourneyStatusResult.m_Item1 == -1) {

                displayMessage("Unable to retrieve journey status, we will re-try shortly.", "Hmm...");
            }
            else {

                if (data.checkJourneyStatusResult.m_Item1 == 1) {

                    $('#LoadingText').html("Found the Driver,<br>Waiting for a Response (Max 30 Seconds)");

                    if (checkDate == null) {

                        checkDate = new Date(Date.now() + 30000);
                    }
                    else {

                        if (checkDate < Date.now()) {

                            $.ajax({

                                type: "POST",
                                url: API + "setJourneyStatus/",
                                contentType: "application/json; charset=utf-8",
                                dataType: "json",
                                data: JSON.stringify({ "JourneyID": JourneyID, "Status": 5 }),
                                success: function (data, status, jqXHR) {

                                    $('.Screen').hide();
                                    $('#Loading').hide();
                                    $('#DriverList').show();
                                    bookTaxi();
                                    displayMessage("No response from the driver was received, this could be because they're busy.", "Hmm...");
                                    clearInterval(checkJourneyStatusTicker);
                                },
                                error: function () {
                                    clearInterval(checkJourneyStatusTicker);
                                    showCommError();
                                }
                            });
                        }
                    }
                }
                else if (data.checkJourneyStatusResult.m_Item1 == 2) {

                    $('#MadeConnection').show();

                    $('#tbAddrFromLn1').val("");
                    $('#tbAddrFromLn2').val("");
                    $('#tbAddrFromLn3').val("");

                    $('#tbAddrToLn1').val("");
                    $('#tbAddrToLn2').val("");
                    $('#tbAddrToLn3').val("");

                    $('#ddlPeople').val("1");
                    $('#ddlDisAcc').val("No");

                    $('#tbRec').val("");
                    $('#tbNotes').val("");

                    $('#cancelJob').show();

                    if (curStatus != data.checkJourneyStatusResult.m_Item1) {
                        curStatus = data.checkJourneyStatusResult.m_Item1;
                        displayMessage("The driver has accepted the job.", "Driver Update");
                    }

                    $.ajax({

                        url: API + "getJourneyDriverLatLon/" + JourneyID,
                        success: function (data) {

                            if (Map == null) {

                                Map = L.map("MadeConnection", { zoomControl: false });

                                L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
                                    attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> Contributors'
                                }).addTo(Map);
                            }

                            if (driverMarker != null) {

                                Map.removeLayer(driverMarker);
                            }

                            driverMarker = L.marker([data.getJourneyDriverLatLonResult.m_Item1, data.getJourneyDriverLatLonResult.m_Item2], { icon: driverMarkerIcon }).addTo(Map);
                            Map.setView([data.getJourneyDriverLatLonResult.m_Item1, data.getJourneyDriverLatLonResult.m_Item2], 16);

                            Map.invalidateSize();

                        },

                        error: function (data) { Map.removeLayer(driverMarker); }
                    });

                    $('#Loading').hide();
                }

                else if (data.checkJourneyStatusResult.m_Item1 == 3) {

                    clearInterval(checkJourneyStatusTicker);

                    //$('#Home').show();
                    $('#DriverList').show();

                    $('#BackBar').show();
                    $('#BackBar').attr('onclick', 'goToPage("DriverList", "Home");');

                    $('#Loading').hide();
                    displayMessage("The driver has unfortunately declined your request.", "Request Declined.");
                }
                else if (data.checkJourneyStatusResult.m_Item1 == 4) {

                    clearInterval(checkJourneyStatusTicker);

                    $('#Home').show();
                    $('#Loading').hide();
                    $('#MadeConnection').hide();
                    $('#cancelJob').hide();
                }
                else {

                    if (curStatus != data.checkJourneyStatusResult.m_Item1) {

                        navigator.notification.vibrate(500);
                        navigator.notification.beep(1);

                        curStatus = data.checkJourneyStatusResult.m_Item1;

                        switch (curStatus) {
                            case 6:
                                displayMessage("The driver will be 5 minutes.", "Driver Update");

                                DateTime = new Date();
                                DateTime.setMinutes(DateTime.getMinutes() - 20);

                                break;
                            case 7:
                                displayMessage("The driver will be 10 minutes.", "Driver Update");

                                DateTime = new Date();
                                DateTime.setMinutes(DateTime.getMinutes() - 25);

                                break;
                            case 8:
                                displayMessage("The driver will be 15 minutes.", "Driver Update");

                                DateTime = new Date();
                                DateTime.setMinutes(DateTime.getMinutes() - 30);

                                break;
                            case 9:
                                displayMessage("The driver has arrived outside.", "Driver Update");

                                DateTime = new Date();
                                DateTime.setMinutes(DateTime.getMinutes() - 15);

                                break;
                            case 10:
                                displayMessage("The driver is en-route.", "Driver Update");

                                DateTime = new Date();
                                DateTime.setMinutes(DateTime.getMinutes() - 15);

                                break;
                        }
                    }

                    $.ajax({

                        url: API + "getJourneyDriverLatLon/" + JourneyID,
                        success: function (data) {

                            if (driverMarker != null) {

                                Map.removeLayer(driverMarker);
                            }

                            driverMarker = L.marker([data.getJourneyDriverLatLonResult.m_Item1, data.getJourneyDriverLatLonResult.m_Item2], { icon: driverMarkerIcon }).addTo(Map);
                            Map.setView([data.getJourneyDriverLatLonResult.m_Item1, data.getJourneyDriverLatLonResult.m_Item2], 16);

                            Map.invalidateSize();

                        },

                        error: function (data) { Map.removeLayer(driverMarker); }
                    });
                }
            }
            //}
        },

        error: function () {
            showCommError();
        }
    });
}

function getGPS() {

    var suc = function (p) {

        Lat = p.coords.latitude;
        Lon = p.coords.longitude;

        getRoadFromGPS(Lat, Lon);
    };

    var err = function () {

        Lat = null;
        Lon = null;
    };

    if (device.platform == "Win32NT") {
        intel.xdk.geolocation.getCurrentPosition(suc, err);
    }
    else {
        navigator.geolocation.getCurrentPosition(suc, err, { maximumAge: 3000, timeout: 5000, enableHighAccuracy: true });
    }
}

function showCommError() {
    $('#BackBar').hide();
    hardError("Comm Error", "A communication error occurred with the API server, if this problem persists please contact Submarine.");
}

function saveSettings() {

    telephoneNumber = $('#tbTelephone').val();

    homeAddressLn1 = $('#tbTMHAddrLn1').val();
    homeAddressLn2 = $('#tbTMHAddrLn2').val();
    homeAddressLn3 = $('#tbTMHAddrLn3').val();

    updateSettings(telephoneNumber, $('#tbTMHAddrLn1').val(), $('#tbTMHAddrLn2').val(), $('#tbTMHAddrLn3').val());

    goToPage("SettingsScreen", "Home");
}

function updateSettings(Telephone, AddrLn1, AddrLn2, AddrLn3) {
    $.ajax({

        type: "POST",
        url: API + "updateClientSettings/",
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        data: JSON.stringify({ "UID": UUID, "Telephone": Telephone, "AddrLn1": AddrLn1, "AddrLn2": AddrLn2, "AddrLn3": AddrLn3 }),

        error: function () {
            displayMessage("A problem occurred whilst attempting to update your settings, please try again shortly.", "Uh Oh!");
        }
    });
}

function getTelephone() {
    $.ajax({

        url: API + "getClientTelephone/" + UUID,
        success: function (data) {

            telephoneNumber = data.getClientTelephoneResult;
        },

        error: function (data) { showCommError(); }
    });
}

function getAddress() {
    $.ajax({

        url: API + "getClientAddress/" + UUID,
        success: function (data) {

            homeAddressLn1 = data.getClientAddressResult.m_Item1;
            homeAddressLn2 = data.getClientAddressResult.m_Item2;
            homeAddressLn3 = data.getClientAddressResult.m_Item3;
        },

        error: function (data) { showCommError(); }
    });
}

function selectPOI(POI) {

    $.ajax({
        url: API + "getPOIByID/" + POI,
        success: function (data) {

            $('#tbAddrToLn1').val(data.getPOIByIDResult.m_Item4)
            $('#tbAddrToLn2').val(data.getPOIByIDResult.m_Item5)
            $('#tbAddrToLn3').val(data.getPOIByIDResult.m_Item6)

            goToPage('POI', 'FindATaxi');
        },
        error: function () {
            showCommError();
        }
    });
}

function selectAddress(Address) {

    $.ajax({
        url: API + "getAddressByID/" + Address + "/" + ClientID + "/",
        success: function (data) {

            $('#addrName').html(data.getAddressByIDResult.m_Item2);

            $('#tbViewAddrLn1').val(data.getAddressByIDResult.m_Item3)
            $('#tbViewAddrLn2').val(data.getAddressByIDResult.m_Item4)
            $('#tbViewAddrLn3').val(data.getAddressByIDResult.m_Item5)

            goToPage('AddressBook', 'ViewAddress');

            $('#btnGoToAddress').attr('onclick', 'goToAddress("' + data.getAddressByIDResult.m_Item1 + '");');
            $('#btnDeleteAddress').attr('onclick', 'deleteAddress("' + data.getAddressByIDResult.m_Item1 + '");');

            $('#BackBar').show();
            $('#BackBar').attr('onclick', 'goToPage("ViewAddress", "AddressBook");');
        },
        error: function () {
            showCommError();
        }
    });
}

function goToAddress(Address) {

    $('#BackBar').hide();

    $.ajax({
        url: API + "getAddressByID/" + Address + "/" + ClientID + "/",
        success: function (data) {

            $('#tbAddrToLn1').val(data.getAddressByIDResult.m_Item3)
            $('#tbAddrToLn2').val(data.getAddressByIDResult.m_Item4)
            $('#tbAddrToLn3').val(data.getAddressByIDResult.m_Item5)

            goToPage('ViewAddress', 'FindATaxi');
        },
        error: function () {
            showCommError();
        }
    });
}

function addAddress() {

    if ($('#tbAddAddrName').val().length > 0 && $('#tbAddAddrLn1').val().length > 0 && $('#tbAddAddrLn2').val().length > 0 && $('#tbAddAddrLn3').val().length > 0) {

        $.ajax({

            type: "POST",
            url: API + "addAddress/",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: JSON.stringify({ "Name": $('#tbAddAddrName').val(), "Ln1": $('#tbAddAddrLn1').val(), "Ln2": $('#tbAddAddrLn2').val(), "Ln3": $('#tbAddAddrLn3').val(), "ClientID": ClientID }),

            success: function () {

                goToPage('AddAddress', 'AddressBook');
            },

            error: function () {
                showCommError();
            }
        });
    }
    else {
        displayMessage("All fields on this form are required, please complete them and re-submit.", "Missing Fields");
    }
}

function deleteAddress(Address) {

    console.log(Address);

    $.ajax({

        type: "POST",
        url: API + "deleteAddress/",
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        data: JSON.stringify({ "Address": Address, "ClientID": ClientID }),

        success: function () {

            goToPage('ViewAddress', 'AddressBook');
        },

        error: function () {
            showCommError();
        }
    });
}

function RegisterDevice() {

    $('#Loading').show();
    $('#LoadingText').html("Registering Device");

    if ($('#tbRegTelephone').val().length > 0 && $('#tbRegName').val().length > 0) {

        $.ajax({

            type: "POST",
            url: API + "registerDevice/",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: JSON.stringify({ "UUID": UUID, "TelephoneNumber": $('#tbRegTelephone').val(), "Name": $('#tbRegName').val() }),

            success: function () {

                $('#RegisterDevice').hide();
                $('#ActivateDevice').show();

                $('#Loading').hide();
            },

            error: function () {
                showCommError();
            }
        });
    }
    else {
        $('#Loading').hide();
        displayMessage("Please enter your mobile phone number and a name, you'll receive an activation code shortly.", "Missing Field");
    }
}

function ActivateDevice() {

    $('#Loading').show();
    $('#LoadingText').html("Activating Device");

    if ($('#tbActivationCode').val().length == 4) {

        $.ajax({

            type: "POST",
            url: API + "activateDevice/",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: JSON.stringify({ "UUID": UUID, "Code": $('#tbActivationCode').val() }),

            success: function (data) {

                if (data.activateDeviceResult == 1) {

                    // Invalid Activation Code
                    displayMessage("Unfortunately your activation code is incorrect, please try again or contact Submarine.", "Invalid Code");
                    $('#Loading').hide();
                }
                else {

                    $('#ActivateDevice').hide();

                    userMarkerIcon = L.icon({

                        iconUrl: 'assets/Start.png',

                        iconSize: [75, 75],
                        iconAnchor: [37.5, 75]
                    });

                    destinationMarkerIcon = L.icon({

                        iconUrl: 'assets/End.png',

                        iconSize: [75, 75],
                        iconAnchor: [37.5, 75]
                    });

                    driverMarkerIcon = L.icon({

                        iconUrl: 'assets/Taxi.png',

                        iconSize: [75, 75],
                        iconAnchor: [37.5, 75]
                    });

                    $('#LoadingText').html("Checking for Previous Jobs");

                    getTelephone();
                    getAddress();
                    isPreviousJob();
                }
            },

            error: function () {
                showCommError();
            }
        });
    }
    else {
        $('#Loading').hide();
        displayMessage("Please enter your 4 digit activation code, received via SMS.", "Missing Field");
    }
}

function cancelJob() {

    clearInterval(checkJourneyStatusTicker);

    $('#CancellationReason').show();

    $('input[type=radio]:checked').val("Other");
    $('#tbReason').val();

    $('#MadeConnection').hide();
    $('#cancelJob').hide();

    $('#Loading').hide();
}

function completeCancellation() {

    var Reason = null;

    console.log($('input[name=cancellationRadio]:checked').val());

    if ($('input[type=radio]:checked').val() == "Other") {

        if ($('#tbReason').val().length == 0) {
            displayMessage("Please include a reason as to why you'd like to cancel this job.", "Missing Fields");
        }
        else {
            var Reason = $('#tbReason').val();
        }
    }
    else {
        Reason = $('input[type=radio]:checked').val();
    }

    if (Reason != null) {

        $.ajax({

            type: "POST",
            url: API + "cancelJourney/",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: JSON.stringify({ "JourneyID": JourneyID, "Reason": Reason }),

            success: function (data, status, jqXHR) {

                clearInterval(checkJourneyStatusTicker);

                $('#CancellationReason').hide();
                $('#Home').show();

                displayMessage("You have successfully cancelled the requested job.", "Job Cancelled");
            },
            error: function () {
                clearInterval(checkJourneyStatusTicker);
                showCommError();
            }
        });
    }
}

function cancelCancellation() {

    $('#CancellationReason').hide();
    $('#tbReason').val("");

    $('#MadeConnection').show();
    $('#cancelJob').show();

    checkJourneyStatusTicker = setInterval(checkJourneyStatus, 1000);
}

function advancedBooking() {

    if ($('#tbPickupDate').val().length == 10 && $('#tbPickupTime').val().length == 5) {

        $.ajax({

            type: "POST",
            url: API + "createJourney/",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: JSON.stringify({ "ClientUID": UUID, "FrLn1": FromLn1, "FrLn2": FromLn2, "FrLn3": FromLn3, "ToLn1": ToLn1, "ToLn2": ToLn2, "ToLn3": ToLn3, "Rec": Rec, "Notes": Notes, "DriverID": "", "People": NumPeople, "DisAcc": DisabilityAccess, "Date": $('#tbPickupDate').val(), "Time": $('#tbPickupTime').val() }),
            success: function (data, status, jqXHR) {

                if (data.createJourneyResult < 1) {

                    if (data.createJourneyResult == 0.1) {

                        $('#Home').show();
                        $('#Loading').hide();
                        displayMessage("Unfortunately we could not create your job due to a permission or status error.", "Hmm...");
                    }
                    else if (data.createJourneyResult == 0.2) {

                        // TODO: Hmmm?

                        $('#Home').show();
                        $('#Loading').hide();
                        displayMessage("Unfortunately there are no available drivers at this time. This could be because they're all busy, please try again shortly.", "Uh Oh!");
                    }
                    else {

                        $('#Home').show();
                        $('#Loading').hide();
                        displayMessage("Something went wrong but we don't know why, please try again shortly.", "An error occurred.");
                    }
                }
                else {

                    $('#tbPickupDate').val("");
                    $('#tbPickupTime').val("");

                    $('#tbAddrFromLn1').val("");
                    $('#tbAddrFromLn2').val("");
                    $('#tbAddrFromLn3').val("");

                    $('#tbAddrToLn1').val("");
                    $('#tbAddrToLn2').val("");
                    $('#tbAddrToLn3').val("");

                    $('#ddlPeople').val("1");
                    $('#ddlDisAcc').val("No");

                    $('#tbRec').val("");
                    $('#tbNotes').val("");

                    goToPage('AdvancedBooking', 'Home');
                    displayMessage("Your booking has been created, we will let you know when a driver accepts it.", "Success");

                    $('#BackBar').hide();
                }
            },
            error: function () {
                showCommError();
            }
        });
    }
    else {
        displayMessage("Please complete both fields of this form.", "Missing Fields");
    }
}

function selectBooking(ID) {

    $('#Loading').show();
    $('#LoadingText').html("Getting Booking Details");

    $('#BookingDetails').show();
    $('#Bookings').hide();

    $('#BackBar').attr('onclick', 'goToPage("BookingDetails", "Bookings");');

    $.ajax({
        url: API + "getBookingDetails/" + ID,
        success: function (data) {

            $('#BookingDetails').html("<b>From: </b>" + data.getBookingDetailsResult.m_Item1 + "<br><br>" + "<b>To: </b>" + data.getBookingDetailsResult.m_Item2 + "<br><br>" + "<b>When: </b>" + data.getBookingDetailsResult.m_Item3 + "<br><br>" + "<b>Driver: </b>" + data.getBookingDetailsResult.m_Item4 + "<br><br>" + "<div class='Button2' style='background: linear-gradient(to bottom, #DC6162 0%, #D43A3C 100%);' onclick=\"cancelBooking('" + ID + "');\">CANCEL BOOKING</div>");
            $('#Loading').hide();
        },
        error: function () {
            $('#Loading').hide();
            goToPage("BookingDetails", "Bookings");
            displayMessage("Something went wrong whilst trying to obtain your booking details, please try again later.", "Hmm...");
        }
    });
}

function cancelBooking(ID) {

    $.ajax({

        type: "POST",
        url: API + "cancelBooking/",
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        data: JSON.stringify({ "BookingID": ID, "DriverOrClient": "Client" }),
        success: function (data, status, jqXHR) {

            goToPage("BookingDetails", "Bookings");
            displayMessage("Your booking has been cancelled.", "Success");
        },
        error: function () {
            goToPage("BookingDetails", "Bookings");
            displayMessage("Something went wrong whilst trying to cancel your booking, please try again later.", "Hmm...");
        }
    });
}